﻿package {
	import flash.utils.ByteArray;
	import flash.display.MovieClip;
	import flash.events.*;
	import Effects;
	import Selection;
	import Script;

	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.SoundMixer;

	public class Field extends MovieClip {

		public var size:Object = { x:41, y: 31 };
		public var dots:Array = new Array();
		public var base:Selection = null;
		public var cache:Object = new Object();
		public var channel:SoundChannel = null;
		public var fx:Effects = new Effects();
		public var script:Script = null;
		private var endCallback:Function = null;
		
		private var soundtrack:Soundtrack = null;
		private var spectrumBytes:ByteArray = new ByteArray();
		public var soundSpectrum:Array = new Array( 4 );


		public function Field( width:Number, height:Number, stageWidth:Number, stageHeight:Number ) {
			this.size.x = width;
			this.size.y = height;
			
			var size:Number = stageWidth / this.size.x / 2;
			this.x = stageWidth / 2;
			this.y = stageHeight / 2 - stageHeight / this.size.y;
			
			// create
			for( var x = 0; x < this.size.x; x++ ) {
				this.dots[x] = new Array();
				for( var y = 0; y < this.size.y; y++ ) {
					var dot:MovieClip = new MovieClip();
					dot.graphics.beginFill( 0xffffff );
					dot.graphics.drawCircle( 0, 0, size );
					dot.x = x*size*2 + size - stageWidth / 2;
					dot.y = y*size*2 + size - stageHeight / 2;
					dot.scaleX = dot.scaleY = 0;
					this.addChild( dot );
					this.dots[x][y] = dot;
				}
			}

			this.base = new Selection( this );
			this.script = new Script( this );
			this.soundtrack = new Soundtrack();
		}
		
		
		public function begin( endCallback:Function = null ) {
			this.endCallback = endCallback;
			this.addEventListener( Event.ENTER_FRAME, this.update );
			this.channel = this.soundtrack.play( 0 );
			this.channel.addEventListener( Event.SOUND_COMPLETE, this.end );
		}
		
		
		private function end( e:Event ) {
			removeEventListener( Event.ENTER_FRAME, this.update );
			if( this.endCallback != null ) {
				this.endCallback();
			}
		}


		private function update( e:Event ) {
			// Calculate sound spectrum (left channel only)
			var spectrumSum:Number = 0;
			var spectrumWidth:Number = 256 / this.soundSpectrum.length;
			SoundMixer.computeSpectrum( this.spectrumBytes, true, 0);
			for (var i:int = 1; i <= 256; i++) {
				spectrumSum += this.spectrumBytes.readFloat();
				if( (i % spectrumWidth) == 0 ) {
					this.soundSpectrum[ (i / spectrumWidth)-1 ] = spectrumSum / spectrumWidth;
					spectrumSum = 0;
				}
			}

			this.script.update( this.channel.position );
		}
	}
}